/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSARC_H
#define PAPYRUSARC_H

#include <cmath>

#include <papyrus/shape.h>

/**
 * \page guide_arc Arcs
 * An arc is drawn centered on the (x,y) coordinate with a specified radius
 * and start/stop angles.
 *
 * The following contains an example that will add a yellow arc with a black
 * border to a canvas. The blue lines represent the
 * x and y axes, and show through since an alpha value of 0.9 is used.
 *
 * \image html arc.png "Arc drawn by example code below"
 *
 * \dontinclude factory.cpp
 * \skip example_arc
 * \until }
 *
 */

namespace Papyrus
{

  /**
   * An arc is drawn centered on the (x,y) coordinate with a specified radius
   * and start/stop angles.
   *
   * \image html arc.png "Arc"
   *
   * @author Rick L Vinyard Jr
   */
  class Arc : public Shape
  {
    protected:

      Arc( double radius,
           double angle_start,
           double angle_end,
           Cairo::RefPtr<Cairo::Pattern> fill
         );

      Arc( double radius,
           double angle_start,
           double angle_end,
           Cairo::RefPtr<Cairo::Pattern> fill,
           const LineStyle& outline
         );

    public:
      typedef PapyrusSmartPointer<Arc> pointer;

      /** Creates an arc with the specified start angle, stop angle and radius. */
      static Arc::pointer create( double radius = 1.0,
                                  double angle_start = 0.0,
                                  double angle_end = M_PI / 2.0,
                                  Cairo::RefPtr<Cairo::Pattern> fill = Cairo::RefPtr<Cairo::Pattern>()
                                );

      /** Creates an arc with the specified start angle, stop angle and radius. */
      static Arc::pointer create( double radius,
                                  double angle_start,
                                  double angle_end,
                                  Cairo::RefPtr<Cairo::Pattern> fill,
                                  const LineStyle& outline
                                );

      /** Destructor */
      ~Arc();

      /** Returns the radius of this arc */
      double radius();

      /** Sets the radius to r */
      void set_radius( double r );

      /** Sets the start angle of the arc. */
      double start_angle();

      /** Sets the end angle of the arc. */
      double end_angle();

      /** Set both start and stop angles in a single method. */
      void set_angles( double start, double end );

      /** Draws on the provided context */
      virtual void draw_shape( Cairo::RefPtr<Cairo::Context> cairo );

      /**
       * Reimplements the draw_outline() method since the draw_shape() method is
       * not sufficient to complete the outline path for arcs less than full circles.
       */
      virtual void draw_outline( Cairo::RefPtr<Cairo::Context> cairo );

      /** Signal emitted when the radius is changed */
      sigc::signal<void> signal_radius();

      /** Signal emitted when the start or end angle is changed */
      sigc::signal<void> signal_angle();

      PAPYRUS_CLASS_NAME("Arc");

      PAPYRUS_CLONE_METHOD( Arc );

    protected:

      /** The radius of the arc */
      double m_radius;

      double m_angle_start;

      double m_angle_end;

      /**
       * Reimplements the need_redraw() method to calculate the extents.
       * Then calls Drawable::need_redraw().
       */
      virtual void need_redraw();

      void calculate_extents();

      /** Signal emitted when the radius is changed */
      sigc::signal<void> m_signal_radius;

      /** Signal emitted when the start or end angle is changed */
      sigc::signal<void> m_signal_angle;

  };

}

#endif
