/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSCANVAS_H
#define PAPYRUSCANVAS_H

#include <papyrus/group.h>

#include <cairomm/cairomm.h>

/**
 * \mainpage Papyrus Library Programmer's Guide
 *
 * This is the programmer's guide to the Papyrus library.
 *
 * \section guide_canvas Guide to the Papyrus Canvas
 *
 * The following provides a brief overview of the papyrus canvas. For more specific details
 * see the detailed class documentation in the <a href="hierarchy.html">Class Hierarchy</a>
 * or <a href="annotated.html">Class List</a> sections.
 *
 * <h2><a href="quick_start.html">Quick Start Guide to Papyrus</a></h2>
 *
 * <h2>Drawing</h2>
 * The following contains a list of drawables currently in Papyrus. Each of the
 * links below leads to a page that illustrates images that were generated by
 * Papyrus, and also include the code that actually generated the example images.
 * - Papyrus::Renderable (Base class for rendering items into a cairomm context)
 * - Papyrus::Drawable (Base class for drawing items with affine transforms)
 *   - Papyrus::Shape (Base class for shape items with outlines and fills)
 *     - \ref guide_arc
 *     - \ref guide_bezierline
 *     - \ref guide_beziergon
 *     - \ref guide_circle
 *     - \ref guide_polyline
 *     - \ref guide_polygon
 *     - \ref guide_rectangle
 *     - \ref guide_regularpolygon
 *     - \ref guide_text
 *   - \ref guide_group
 *     - \ref guide_boxed
 *   - \ref guide_image "Images"
 *   - \ref guide_pixbuf
 *
 * <h2>ViewBoxes</h2>
 * ViewBoxes are conceptually the same as the viewBox attribute in SVG.
 * - \ref guide_viewbox
 *
 */

/**
 * \page quick_start Quick Start Guide to Papyrus
 *
 * <br />
 * This is intended to be a very succinct guide to papyrus, and is \b not
 * intended as a general tutorial.
 * <br />
 *
 * <h2>Concept #1: The canvas</h2>
 * Papyrus is structured around the concept of a model-view separation. In the Papyrus library,
 * there are no views... only models, including Papyrus::Canvas itself. <i>(Read on to find out
 * where the views are)</i>
 * <br /> <br />
 *
 * <h2>Concept #2: Drawables</h2>
 * Drawables are added to a Papyrus::Canvas. Most drawables inherit from Papyrus::Shape.
 * <br /> <br />
 *
 * <h2>Concept #3: Rendering <i>(or moving from models to eye candy)</i></h2>
 * A papyrus model is made 'tangible' by rendering into a cairo context. Papyrus::Canvas and most
 * other objects in Papyrus inherit from Papyrus::Renderable which has a \c render()
 * method. Thus, rendering into a cairomm context is the way to view a Papyrus::Canvas.
 * <br /> <br />
 *
 * <h2>Concept #4: Cairo Contexts</h2>
 * In cairo, a context is created from a surface. Cairo supports numerous surfaces including:
 *   - Gdk and Gdkmm widgets
 *   - PDF Surfaces
 *   - PostScript Surfaces
 *   - Image Surfaces (in-memory buffers and PNG)
 *   - SVG Surfaces
 *   - Glitz (OpenGL) Surfaces
 *
 * This quick-start won't go into the specifics of cairomm surfaces (I'll leave that to the cairomm
 * library), but suffice it to say that a Papyrus::Renderable (including the Papyrus::Canvas) is rendered
 * into a cairomm context which results in drawing a papyrus object to any of the above surfaces.
 * <br /> <br />
 *
 * <h2>Concept #5: Viewports </h2>
 * PapyrusGtk::Viewport is a Gtkmm widget that serves as a view on a Papyrus::Canvas model, and
 * creates a cairomm context 'behind the scenes'. This provides a viewport into the world of the
 * canvas model you have created.
 *
 * Viewports are only one of many ways to display a Papyrus model in a Gtkmm widget.
 * <br /> <br />
 *
 * <h2>Concepts #6, #7 and #8: Smart Pointers, \c pointer and \c create() </h2>
 * An early design decision was made to focus on the use of smart pointers as defined in the ANSI C++
 * committee's \c tr1 (Technical Report 1), which as of this writing has been accepted by the committee
 * and recommended to ISO for incorporation as the first revision to the C++ standard. A reference
 * implementation can be found in the boost library or in the \c std::tr1 namespace of gcc 4.0 or
 * greater.
 *
 * Two key concepts are the \c pointer typedef and the \c create method defined in each Papyrus class.
 *
 * The \c pointer typedef is defined within each class and is typedefed to a smart pointer
 * to that class. Thus, \c Papyrus::Rectangle::pointer is a smart pointer to a rectangle,
 * and \c Papyrus::Circle::pointer is a smart pointer to a circle.
 *
 * Each class also has static \c create() methods with parameters that are identical to
 * the class' constructors. The \c create() method returns a smart pointer to that object,
 * similar to the way the \c new operator returns a pointer. Since the \c create() method is
 * static, you do not need an instance of the class to call it; \c Classname::create() is sufficient.
 *
 * Naturally, the smart pointer returned from \c create() can be assigned to the class' \c pointer
 * type, which allows you to use a syntax such as:
 * \code Classname::pointer my_variable = Classname::create(); \endcode
 * <br /> <br />
 *
 * <h2>Putting it all together</h2>
 * <br />
 * Create a PapyrusGtk viewport
 * \code PapyrusGtk::Viewport my_viewport; \endcode
 * <br />
 *
 * Set the viewport to an initial size of 200 x 200 and center the view
 * \code
 * my_viewport.canvas->set_size(200, 200);
 * my_viewport.set_scroll_point( PapyrusGtk::SCROLL_TO_CENTER );
 * \endcode
 * <br />
 *
 * Create a rectangle of length 100 and width 50
 * \code Papyrus::Rectangle::pointer my_rectangle = Papyrus::Rectangle::create(100, 50); \endcode
 * <br />
 *
 * Color your rectangle red and make it slightly transparent by setting alpha to 0.9
 * \code my_rectangle->set_fill( Papyrus::RGBA(1.0, 0.0, 0.0, 0.9) ); \endcode
 * <br />
 *
 * Add your red (and slightly transparent) rectangle to the viewport's canvas
 * \code my_viewport.canvas()->add( my_rectangle ); \endcode
 * <br />
 *
 * At this point you have a Gtkmm widget named \c my_viewport that you can incorporate
 * into your program as you would any other Gtkmm widget by adding it to a container, window, etc.
 * \code
 * Gtk::Window my_window;
 * my_window.add( my_viewport );
 * \endcode
 * <br />
 *
 * Since this isn't a tutorial on Gtkmm (they have a fine one
 * <a href="http://gtkmm.org/docs/gtkmm-2.4/docs/tutorial/html/">here</a>) I'll bring this
 * quick start to an end.
 * <br /> <br />
 *
 */

namespace Papyrus {

  /**
  @author Rick L Vinyard Jr
  */
  class Canvas: public Group {

    public:
      typedef PapyrusSmartPointer<Canvas> pointer;

      Canvas(double w=0.0, double h=0.0);

      static Canvas::pointer create(double w=0.0, double h=0.0);

      virtual ~Canvas();

      double size_w();

      double size_h();

      void size( double& w, double& h );

      void set_size_w( double w );

      void set_size_h( double h );

      void set_size( double w, double h );

      sigc::signal<void> signal_size();

      void set_background( Cairo::RefPtr<Cairo::Pattern> background );

      Cairo::RefPtr<Cairo::Pattern> get_background();

      PAPYRUS_CLASS_NAME("Canvas");

      PAPYRUS_CLONE_METHOD( Canvas );

    protected:
      double m_width, m_height;
      Cairo::RefPtr<Cairo::Pattern> m_background;
      sigc::signal<void> m_signal_size;

      virtual void draw( Cairo::RefPtr<Cairo::Context> cairo );

  };

}

#endif
