/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "drawable.h"
#include "group.h"

namespace Papyrus
{

  Drawable::Drawable( double tx, double ty, double sx, double sy, double r )
      : Renderable(),
      m_parent( NULL ),
      m_drawing_parent( NULL ),
      m_visible( true ),
      m_tx( tx ),
      m_ty( ty ),
      m_sx( sx ),
      m_sy( sy ),
      m_r( r ),
      m_skewx( 0.0 ),
      m_skewy( 0.0 ),
      m_selectable( true ),
      m_centroid_x( 0.0 ),
      m_centroid_y( 0.0 ),
      m_exclude_from_extents( false ),
      m_regenerate_matrix( true ),
      m_extents(-INFINITY, -INFINITY, INFINITY, INFINITY)
  {
    if ( m_sx < 0.0 )
      m_sx = 1.0;
    if ( m_sy < 0.0 )
      m_sy = 1.0;
  }

  Drawable::~Drawable()
  {
    if ( m_parent != NULL )
      m_parent->remove( this );
    if ( m_drawing_parent != NULL )
      m_drawing_parent->remove( this );
  }

  bool Drawable::is_visible()
  {
    return m_visible;
  }

  void Drawable::show()
  {
    if ( m_visible ) return;
    m_visible = true;
    this->need_redraw();
  }

  void Drawable::hide()
  {
    if ( ! m_visible ) return;
    m_visible = false;
    this->need_redraw();
  }

  const Cairo::Matrix& Drawable::matrix()
  {
    this->recalculate_matrix();
    return m_matrix;
  }

  Cairo::Matrix Drawable::global_matrix()
  {
    Cairo::Matrix this_matrix = this->matrix();
    if ( m_parent != NULL ) {
      Cairo::Matrix parent_matrix = m_parent->global_matrix();
      cairo_matrix_multiply( &this_matrix, &parent_matrix, &this_matrix );
    }
    return this_matrix;
  }

  void Drawable::global_to_local( double& x, double& y )
  {
    Cairo::Matrix matrix = this->global_matrix();
    cairo_matrix_transform_point( &matrix, &x, &y );
  }

  void Drawable::local_to_global( double& x, double& y )
  {
    Cairo::Matrix matrix = this->global_matrix();
    cairo_matrix_invert( &matrix );
    cairo_matrix_transform_point( &matrix, &x, &y );
  }

  void Drawable::distance_global_to_local( double& x, double& y )
  {
    Cairo::Matrix matrix = this->global_matrix();
    cairo_matrix_transform_distance( &matrix, &x, &y );
  }

  void Drawable::distance_local_to_global( double& x, double& y )
  {
    Cairo::Matrix matrix = this->global_matrix();
    cairo_matrix_invert( &matrix );
    cairo_matrix_transform_distance( &matrix, &x, &y );
  }

  double Drawable::get_translate_x()
  {
    return m_tx;
  }

  double Drawable::get_translate_y()
  {
    return m_ty;
  }

  void Drawable::get_translate( double& tx, double& ty )
  {
    tx = m_tx;
    ty = m_ty;
  }

  void Drawable::set_translate_x( double tx )
  {
    m_tx = tx;
    this->need_redraw();
    m_signal_xy.emit();
  }

  void Drawable::set_translate_y( double ty )
  {
    m_ty = ty;
    this->need_redraw();
    m_signal_xy.emit();
  }

  void Drawable::set_translate( double tx, double ty )
  {
    m_tx = tx;
    m_ty = ty;
    this->need_redraw();
    m_signal_xy.emit();
  }

  void Drawable::translate_x( double tx )
  {
    m_tx = m_tx + tx;
    this->need_redraw();
    m_signal_xy.emit();
  }

  void Drawable::translate_y( double ty )
  {
    m_ty = m_ty + ty;
    this->need_redraw();
    m_signal_xy.emit();
  }

  void Drawable::translate( double tx, double ty )
  {
    m_tx = m_tx + tx;
    m_ty = m_ty + ty;
    this->need_redraw();
    m_signal_xy.emit();
  }

  double Drawable::get_scale_x()
  {
    return m_sx;
  }

  double Drawable::get_scale_y()
  {
    return m_sy;
  }

  void Drawable::get_scale( double& sx, double& sy )
  {
    sx = m_sx;
    sy = m_sy;
  }

  void Drawable::set_scale_x( double sx )
  {
    if ( sx < 0.0 )
      return ;

    m_sx = sx;
    this->need_redraw();
    m_signal_scale.emit();
  }

  void Drawable::set_scale_y( double sy )
  {
    if ( sy < 0.0 )
      return ;

    m_sy = sy;
    this->need_redraw();
    m_signal_scale.emit();
  }

  void Drawable::set_scale( double sx, double sy )
  {
    if ( sx >= 0.0 )
      m_sx = sx;

    if ( sy >= 0.0 )
      m_sy = sy;

    this->need_redraw();
    m_signal_scale.emit();
  }

  void Drawable::set_scale( double s )
  {
    if ( s < 0.0 )
      return ;

    m_sx = s;
    m_sy = s;
    this->need_redraw();
    m_signal_scale.emit();
  }

  void Drawable::scale_x( double s )
  {
    if ( s < 0.0 )
      return ;

    m_sx = m_sx * s;
    this->need_redraw();
    m_signal_scale.emit();
  }

  void Drawable::scale_y( double s )
  {
    if ( s < 0.0 )
      return ;

    m_sy = m_sy * s;
    this->need_redraw();
    m_signal_scale.emit();
  }

  void Drawable::scale( double s )
  {
    if ( s < 0.0 )
      return ;

    m_sx = m_sx * s;
    m_sy = m_sy * s;
    this->need_redraw();
    m_signal_scale.emit();
  }

  void Drawable::scale( double sx, double sy )
  {
    if ( sx >= 0.0 )
      m_sx = m_sx * sx;

    if ( sy >= 0.0 )
      m_sy = m_sy * sy;

    this->need_redraw();
    m_signal_scale.emit();
  }

  double Drawable::get_rotate()
  {
    return m_r;
  }

  void Drawable::set_rotate( double r )
  {
    m_r = r;
    this->need_redraw();
    m_signal_rotation.emit();
  }

  void Drawable::rotate( double r )
  {
    m_r = m_r + r;
    this->need_redraw();
    m_signal_rotation.emit();
  }

  double Drawable::get_skew_x()
  {
    return m_skewx;
  }

  void Drawable::set_skew_x( double skewx )
  {
    m_skewx = skewx;
    this->need_redraw();
    m_signal_skew.emit();
  }

  void Drawable::skew_x( double skewx )
  {
    m_skewx += skewx;
    this->need_redraw();
    m_signal_skew.emit();
  }

  double Drawable::get_skew_y()
  {
    return m_skewy;
  }

  void Drawable::set_skew_y( double skewy )
  {
    m_skewy = skewy;
    this->need_redraw();
    m_signal_skew.emit();
  }

  void Drawable::skew_y( double skewy )
  {
    m_skewy += skewy;
    this->need_redraw();
    m_signal_skew.emit();
  }

  void Drawable::get_skew( double& skewx, double& skewy )
  {
    skewx = m_skewx;
    skewy = m_skewy;
  }

  void Drawable::set_skew( double skewx, double skewy )
  {
    m_skewx = skewx;
    m_skewy = skewy;
    this->need_redraw();
    m_signal_skew.emit();
  }

  void Drawable::skew( double s )
  {
    m_skewx += s;
    m_skewy += s;
    this->need_redraw();
    m_signal_skew.emit();
  }

  void Drawable::skew( double skewx, double skewy )
  {
    m_skewx += skewx;
    m_skewy += skewy;
    this->need_redraw();
    m_signal_skew.emit();
  }


  double Drawable::get_centroid_x()
  {
    return m_centroid_x;
  }

  void Drawable::set_centroid_x( double centroid_x )
  {
    m_centroid_x = centroid_x;
    this->need_redraw();
    m_signal_centroid.emit();
  }

  double Drawable::get_centroid_y()
  {
    return m_centroid_y;
  }

  void Drawable::set_centroid_y( double centroid_y )
  {
    m_centroid_y = centroid_y;
    this->need_redraw();
    m_signal_centroid.emit();
  }

  void Drawable::get_centroid( double& centroid_x, double& centroid_y )
  {
    centroid_x = m_centroid_x;
    centroid_y = m_centroid_y;
  }

  void Drawable::set_centroid( double centroid_x, double centroid_y )
  {
    m_centroid_x = centroid_x;
    m_centroid_y = centroid_y;
    this->need_redraw();
    m_signal_centroid.emit();
  }

  void Drawable::reset_position( )
  {
    m_tx = 0.0;
    m_ty = 0.0;
    m_sx = 1.0;
    m_sy = 1.0;
    m_r = 0.0;
    m_skewx = 0.0;
    m_skewy = 0.0;
    m_centroid_x = 0.0;
    m_centroid_y = 0.0;
    this->need_redraw();
    m_signal_xy.emit();
    m_signal_scale.emit();
    m_signal_rotation.emit();
    m_signal_skew.emit();
    m_signal_centroid.emit();
  }

  void Drawable::render( Cairo::RefPtr<Cairo::Context> cairo )
  {
    double x, y, w, h;

    // We're not going to render if we're invisible
    if ( ! m_visible ) return;

    cairo->save( );

    #if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 2, 0)
    cairo->begin_new_path();
    #else
      cairo->clear_path();
    #endif

    this->recalculate_matrix( );
    cairo->transform( m_matrix );

    if ( m_viewbox && m_viewbox->meet_or_slice() == Viewbox::SLICE ) {
      Cairo::Matrix viewbox_matrix = m_viewbox->transformation_matrix( this->extents() );
      cairo_matrix_invert( &viewbox_matrix );

      m_viewbox->xywh( x, y, w, h );
      cairo_matrix_transform_point( &viewbox_matrix, &x, &y );
      cairo_matrix_transform_distance( &viewbox_matrix, &w, &h );
      cairo->rectangle( x, y, w, h );
      cairo->clip();
    }
    this->draw( cairo );
    cairo->restore( );
  }

  void Drawable::recalculate_matrix( )
  {
    Cairo::Matrix shear_matrix;
    if ( m_regenerate_matrix ) {
      cairo_matrix_init_identity( &m_matrix );
      cairo_matrix_translate( &m_matrix, m_tx + m_centroid_x, m_ty + m_centroid_y );
      cairo_matrix_rotate( &m_matrix, m_r );
      cairo_matrix_translate( &m_matrix, -m_centroid_x, -m_centroid_y );
      cairo_matrix_init_identity( &shear_matrix );
      shear_matrix.xy = tan( m_skewx );
      shear_matrix.yx = tan( m_skewy );
      cairo_matrix_multiply( &m_matrix, &m_matrix, &shear_matrix );
      cairo_matrix_scale( &m_matrix, m_sx, m_sy );
      if ( m_viewbox ) {
        Cairo::Matrix viewbox_matrix = m_viewbox->transformation_matrix( this->extents() );
        cairo_matrix_multiply( &m_matrix, &m_matrix, &viewbox_matrix );
      }

      m_regenerate_matrix = false;
    }
  }

  Region& Drawable::extents( )
  {
    return m_extents;
  }

  Region Drawable::local_extents( )
  {
    Region xformed_extents = this->extents();

    this->recalculate_matrix();

    xformed_extents.transform( m_matrix );

    return xformed_extents;
  }

  Region Drawable::global_extents( )
  {
    Cairo::Matrix inverse_matrix;
    Region xformed_extents = this->extents();

    this->recalculate_matrix();

    if ( m_viewbox ) {
      Cairo::Matrix viewbox_matrix = m_viewbox->transformation_matrix( this->extents() );
      cairo_matrix_multiply( &inverse_matrix, &m_matrix, &viewbox_matrix );
    } else
      inverse_matrix = m_matrix;

    cairo_matrix_invert( &inverse_matrix );

    xformed_extents.transform( m_matrix );

    return xformed_extents;
  }

  Region Drawable::extents_transformed( cairo_matrix_t & globalmatrix )
  {
    Region xformed_extents = this->extents();
    Cairo::Matrix newmatrix;

    this->recalculate_matrix();
    cairo_matrix_multiply( &newmatrix, &m_matrix, &globalmatrix );

    xformed_extents.transform( newmatrix );

    return xformed_extents;
  }

  bool Drawable::inside( double x, double y )
  {
    return this->global_extents().is_inside( x, y );
  }

  bool Drawable::exclude_from_extents()
  {
    return m_exclude_from_extents;
  }

  void Drawable::set_exclude_from_extents(bool b)
  {
    m_exclude_from_extents = b;
    this->need_redraw();
  }

  void Drawable::need_redraw()
  {
    double x, y, w, h;

    m_regenerate_matrix = true;

    Region tmpbox = this->global_extents();
    Region updatebox = m_extentsold;
    m_extentsold = tmpbox;
    updatebox.include( tmpbox );
    updatebox.xywh( x, y, w, h );

    if ( ! m_visible )
      return;

    m_signal_redraw.emit( x, y, w, h );
    m_signal_changed.emit();
  }

  void Drawable::need_redraw_proxy()
  {
    this->need_redraw();
  }

  void Drawable::set_extents( const Region & extents )
  {
    m_extents = extents;
  }

  Viewbox::pointer Drawable::viewbox()
  {
    return m_viewbox;
  }

  void Drawable::set_viewbox( Viewbox::pointer viewbox )
  {
    if ( m_viewbox_connection )
      m_viewbox_connection.disconnect();

    m_viewbox = viewbox;
    if ( m_viewbox )
      m_viewbox_connection = m_viewbox->signal_changed().connect( sigc::mem_fun( *this, &Drawable::need_redraw_proxy ) );
    this->need_redraw();
  }

  void Drawable::set_viewbox( const Viewbox & viewbox )
  {
    Viewbox::pointer vb = Viewbox::create();
    *vb = viewbox;
    this->set_viewbox( vb );
  }

  void Drawable::set_selectable( bool selectable )
  {
    m_selectable = selectable;
  }

  bool Drawable::is_selectable()
  {
    return m_selectable;
  }

  bool Drawable::is_group()
  {
    return false;
  }

  Group* Drawable::parent()
  {
    return m_parent;
  }

  sigc::signal<void, double, double, double, double> Drawable::signal_redraw()
  {
    return m_signal_redraw;
  }

  sigc::signal<void> Drawable::signal_xy()
  {
    return m_signal_xy;
  }

  sigc::signal<void> Drawable::signal_scale()
  {
    return m_signal_scale;
  }

  sigc::signal<void> Drawable::signal_rotation()
  {
    return m_signal_rotation;
  }

  sigc::signal<void> Drawable::signal_skew()
  {
    return m_signal_skew;
  }

  sigc::signal<void> Drawable::signal_selectable()
  {
    return m_signal_selectable;
  }

  sigc::signal<void> Drawable::signal_centroid()
  {
    return m_signal_centroid;
  }

}
