/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "grid.h"

namespace Papyrus
{

  const double Grid::infinity = 1.5e4;

  Grid::Grid(double xmin, double xmax, double ymin, double ymax):
      m_x_min(xmin),
      m_x_max(xmax),
      m_y_min(ymin),
      m_y_max(ymax),
      m_x_interval(0.0),
      m_y_interval(0.0),
      m_enabled_axes(X_AXIS | Y_AXIS),
      m_enabled_borders(0)
  {}

  Grid::pointer Grid::create(double xmin, double xmax, double ymin, double ymax)
  {
    return Grid::pointer( new Grid(xmin, xmax, ymin, ymax) );
  }

  Grid::~Grid()
  {}

  double Grid::x_min( )
  {
    return m_x_min;
  }

  void Grid::set_x_min( double min )
  {
    m_x_min = min;
    this->need_redraw();
  }

  double Grid::x_max( )
  {
    return m_x_max;
  }

  void Grid::set_x_max( double max )
  {
    m_x_max = max;
    this->need_redraw();
  }

  void Grid::x_min_max( double & min, double & max )
  {
    min = m_x_min;
    max = m_x_max;
  }

  void Grid::set_x_min_max( double min, double max )
  {
    m_x_min = min;
    m_x_max = max;
    this->need_redraw();
  }

  double Grid::y_min( )
  {
    return m_y_min;
  }

  void Grid::set_y_min( double min )
  {
    m_y_min = min;
    this->need_redraw();
  }

  double Grid::y_max( )
  {
    return m_y_max;
  }

  void Grid::set_y_max( double max )
  {
    m_y_max = max;
    this->need_redraw();
  }

  void Grid::y_min_max( double & min, double & max )
  {
    min = m_y_min;
    max = m_y_max;
  }

  void Grid::set_y_min_max( double min, double max )
  {
    m_y_min = min;
    m_y_max = max;
    this->need_redraw();
  }

  void Grid::xy_min_max( double & x_min, double & x_max, double & y_min, double & y_max )
  {
    x_min = m_x_min;
    x_max = m_x_max;
    y_min = m_y_min;
    y_max = m_y_max;
  }

  void Grid::set_xy_min_max( double x_min, double x_max, double y_min, double y_max )
  {
    m_x_min = x_min;
    m_x_max = x_max;
    m_y_min = y_min;
    m_y_max = y_max;
    this->need_redraw();
  }

  void Grid::enable_axis( unsigned which )
  {
    unsigned int old_enabled = m_enabled_axes;
    m_enabled_axes |= which;
    if ( old_enabled != m_enabled_axes )
      this->need_redraw();
  }

  void Grid::disable_axis( unsigned which )
  {
    unsigned int old_enabled = m_enabled_axes;
    m_enabled_axes &= ~which;
    if ( old_enabled != m_enabled_axes )
      this->need_redraw();
  }

  void Papyrus::Grid::set_xy_axis_style( const LineStyle & style )
  {
    m_y_axis_style = m_x_axis_style = style;
    this->need_redraw();
  }

  const LineStyle& Papyrus::Grid::x_axis_style( ) const
  {
    return m_x_axis_style;
  }

  void Papyrus::Grid::set_x_axis_style( const LineStyle & style )
  {
    m_x_axis_style = style;
    this->need_redraw();
  }

  const LineStyle& Grid::y_axis_style( ) const
  {
    return m_y_axis_style;
  }

  void Grid::set_y_axis_style( const LineStyle & style )
  {
    m_y_axis_style = style;
    this->need_redraw();
  }

  void Grid::draw( Cairo::RefPtr< Cairo::Context > cairo )
  {
    double start, stop, current;

    // draw interval
    if ( m_x_interval > 0.0 ) {
      if ( m_x_min < 0.0 )
        start = ceil( m_x_min / m_x_interval ) * m_x_interval;
      else
        start = floor( m_x_min / m_x_interval ) * m_x_interval;
      if ( m_x_max < 0.0 )
        stop = ceil( m_x_max / m_x_interval ) * m_x_interval;
      else
        stop = floor( m_x_max / m_x_interval ) * m_x_interval;
      m_x_interval_style.render( cairo );
      for ( current = start; current <= stop; current += m_x_interval ) {
        cairo->move_to( current, m_y_min );
        cairo->line_to( current, m_y_max );
      }
      cairo->stroke();
    }

    if ( m_y_interval > 0.0 ) {
      if ( m_y_min < 0.0 )
        start = ceil( m_y_min / m_y_interval ) * m_y_interval;
      else
        start = floor( m_y_min / m_y_interval ) * m_y_interval;
      if ( m_y_max < 0.0 )
        stop = ceil( m_y_max / m_y_interval ) * m_y_interval;
      else
        stop = floor( m_y_max / m_y_interval ) * m_y_interval;
      m_y_interval_style.render( cairo );
      for ( current = start; current <= stop; current += m_y_interval ) {
        cairo->move_to( m_x_min, current );
        cairo->line_to( m_x_max, current );
      }
      cairo->stroke();
    }

    // draw axes
    if ( m_enabled_axes & X_AXIS && m_x_min <= 0.0 && m_x_max >= 0.0 ) {
      m_x_axis_style.render( cairo );
      cairo->move_to( 0.0, m_y_min );
      cairo->line_to( 0.0, m_y_max );
      cairo->stroke();
    }

    if ( m_enabled_axes & Y_AXIS && m_y_min <= 0.0 && m_y_max >= 0.0  ) {
      m_y_axis_style.render( cairo );
      cairo->move_to( m_x_min, 0.0 );
      cairo->line_to( m_x_max, 0.0 );
      cairo->stroke();
    }

    // draw borders
    if ( m_enabled_borders & TOP ) {
      m_border_style[0].render( cairo );
      cairo->move_to( m_x_min, m_y_min );
      cairo->line_to( m_x_max, m_y_min );
      // Stroke the line if we're not going to continue drawing in the same style
      if ( !(m_enabled_borders & RIGHT && m_border_style[1] == m_border_style[0]) )
        cairo->stroke();
    }

    if ( m_enabled_borders & RIGHT ) {
      // Do we have a style change?
      if ( m_border_style[1] != m_border_style[0] )
        m_border_style[1].render( cairo );
      // Did we leave off at the right spot, or do we need to move to it?
      if ( ! m_enabled_borders & TOP )
        cairo->move_to( m_x_max, m_y_min );
      cairo->line_to( m_x_max, m_y_max );
      // Stroke the line if we're not going to continue drawing in the same style
      if ( !(m_enabled_borders & BOTTOM && m_border_style[2] == m_border_style[1]) )
        cairo->stroke();
    }

    if ( m_enabled_borders & BOTTOM ) {
      // Do we have a style change?
      if ( m_border_style[2] != m_border_style[1] )
        m_border_style[2].render( cairo );
      // Did we leave off at the right spot, or do we need to move to it?
      if ( ! m_enabled_borders & RIGHT )
        cairo->move_to( m_x_max, m_y_max );
      cairo->line_to( m_x_min, m_y_max );
      // Stroke the line if we're not going to continue drawing in the same style
      if ( !(m_enabled_borders & LEFT && m_border_style[3] == m_border_style[2]) )
        cairo->stroke();
    }

    if ( m_enabled_borders & LEFT ) {
      // Do we have a style change?
      if ( m_border_style[3] != m_border_style[2] )
        m_border_style[3].render( cairo );
      // Did we leave off at the right spot, or do we need to move to it?
      if ( ! m_enabled_borders & BOTTOM )
        cairo->move_to( m_x_min, m_y_max );
      cairo->line_to( m_x_min, m_y_min );
      // Should we close the path and make it look nice?
      if ( m_enabled_borders & TOP && m_border_style[0] == m_border_style[3] )
        cairo->close_path();
      cairo->stroke();
    }

  }

  void Grid::enable_border( unsigned which )
  {
    unsigned int old_enabled = m_enabled_borders;
    m_enabled_borders |= which;
    if ( old_enabled != m_enabled_borders )
      this->need_redraw();
  }

  void Grid::disable_border( unsigned which )
  {
    unsigned int old_enabled = m_enabled_borders;
    m_enabled_borders &= ~which;
    if ( old_enabled != m_enabled_borders )
      this->need_redraw();
  }

  void Grid::set_border_style( const LineStyle & style, unsigned which )
  {
    if ( which & TOP )
      m_border_style[ 0 ] = style;
    if ( which & RIGHT )
      m_border_style[ 1 ] = style;
    if ( which & BOTTOM )
      m_border_style[ 2 ] = style;
    if ( which & LEFT )
      m_border_style[ 3 ] = style;
  }

  double Grid::x_interval( )
{
  return m_x_interval;
}

void Grid::set_x_interval( double x_interval )
{
  m_x_interval = x_interval;
  this->need_redraw();
}

const LineStyle & Grid::x_interval_style( )
{
  return m_x_interval_style;
}

void Grid::set_x_interval_style( const LineStyle & style )
{
  m_x_interval_style = style;
  this->need_redraw();
}

double Grid::y_interval( )
{
  return m_y_interval;
}

void Grid::set_y_interval( double y_interval )
{
  m_y_interval = y_interval;
  this->need_redraw();
}

const LineStyle & Grid::y_interval_style( )
{
  return m_y_interval_style;
}

void Grid::set_y_interval_style( const LineStyle & style )
{
  m_y_interval_style = style;
  this->need_redraw();
}

void Grid::xy_interval( double & x_interval, double & y_interval )
{
  x_interval = m_x_interval;
  y_interval = m_y_interval;
}

void Grid::set_xy_interval( double interval )
{
  m_x_interval = m_y_interval = interval;
  this->need_redraw();
}

void Grid::set_xy_interval( double x_interval, double y_interval )
{
  m_x_interval = x_interval;
  m_y_interval = y_interval;
  this->need_redraw();
}

void Grid::xy_interval_style( LineStyle & x_style, LineStyle & y_style )
{
  x_style = m_x_interval_style;
  y_style = m_y_interval_style;
}

void Grid::set_xy_interval_style( const LineStyle & style )
{
  m_x_interval_style = m_y_interval_style = style;
  this->need_redraw();
}

void Grid::set_xy_interval_style( const LineStyle & x_style, const LineStyle & y_style )
{
  m_x_interval_style = x_style;
  m_y_interval_style = y_style;
  this->need_redraw();
}

}
