/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "region.h"

#include <algorithm>

namespace Papyrus {

Region::Region(double x, double y, double w, double h):
        m_x(x),
        m_y(y),
        m_w(w),
m_h(h) {}


Region::~Region() {}

double Region::x() const {
    return m_x;
}
double Region::y() const {
    return m_y;
}
double Region::width() const {
    return m_w;
}
double Region::height() const {
    return m_h;
}

double Region::xywh(double& x, double& y, double& w, double& h) const {
    x = m_x;
    y = m_y;
    w = m_w;
    h = m_h;
}

void Region::set_x(double x) {
    m_x = x;
    m_signal_changed.emit();
}

void Region::set_y(double y) {
    m_y = y;
    m_signal_changed.emit();
}

void Region::set_width(double width) {
    m_w = width;
    m_signal_changed.emit();
}

void Region::set_height(double height) {
    m_h = height;
    m_signal_changed.emit();
}

void Region::set_xywh(double x, double y, double w, double h) {
    m_x = x;
    m_y = y;
    m_w = w;
    m_h = h;
    m_signal_changed.emit();
}

Region& Region::operator=(const Region& other) {
    if (*this == other)
        return *this;
    m_x = other.m_x;
    m_y = other.m_y;
    m_w = other.m_w;
    m_h = other.m_h;
    m_signal_changed.emit();
    return *this;
}

bool Region::operator==(const Region& other) const {
    return (m_x == other.m_x && m_y == other.m_y && m_w == other.m_w && m_h == other.m_h);
}

bool Region::operator<(const Region& other) const {
    return (
               m_x < other.m_x &&
               m_y < other.m_y &&
               m_w > other.m_w &&
               m_h > other.m_h
           );
}

bool Region::operator<=(const Region& other) const {
    return (
               m_x <= other.m_x &&
               m_y <= other.m_y &&
               m_w >= other.m_w &&
               m_h >= other.m_h
           );
}

bool Region::operator>(const Region& other) const {
    return (! (*this <= other ));
}

bool Region::operator>=(const Region& other) const {
    return (! (*this < other ));
}

bool Region::include(const Region& other) {
    bool changed = false;
    if (other.m_x < m_x) {
        m_w = m_x + m_w - other.m_x;
        m_x = other.m_x;
        changed = true;
    }
    if (other.m_y < m_y) {
        m_h = m_y + m_h - other.m_y;
        m_y = other.m_y;
        changed = true;
    }
    if (other.m_x + other.m_w > m_x + m_w) {
        m_w = other.m_x + other.m_w - m_x;
        changed = true;
    }
    if (other.m_y + other.m_h > m_y + m_h) {
        m_h = other.m_y + other.m_h - m_y;
        changed = true;
    }

    if (changed)
        m_signal_changed.emit();

    return changed;
}

bool Region::include(double x, double y) {
    bool changed = false;
    if (x < m_x) {
        m_w += m_x - x;
        m_x = x;
        changed = true;
    } else if (x > m_x + m_w) {
        m_w = x - m_x;
        changed = true;
    }

    if (y < m_y) {
        m_h += m_y - y;
        m_y = y;
        changed = true;
    } else if (y > m_y + m_h) {
        m_h = y - m_y;
        changed = true;
    }

    if (changed)
        m_signal_changed.emit();

    return changed;
}

bool Region::include_x(double x) {
    bool changed = false;
    if (x < m_x) {
        m_w += m_x - x;
        m_x = x;
        changed = true;
    } else if (x > m_x + m_w) {
        m_w = x - m_x;
        changed = true;
    }

    if (changed)
        m_signal_changed.emit();

    return changed;
}

bool Region::include_y(double y) {
    bool changed = false;
    if (y < m_y) {
        m_h += m_y - y;
        m_y = y;
        changed = true;
    } else if (y > m_y + m_h) {
        m_h = y - m_y;
        changed = true;
    }

    if (changed)
        m_signal_changed.emit();

    return changed;
}

bool Region::is_inside( double x, double y ) const {
    return (
               x >= m_x        &&
               x <= m_x + m_w  &&
               y >= m_y        &&
               y <= m_y + m_h
           );
}

bool Region::is_interior( double x, double y ) const {
    return (
               x > m_x        &&
               x < m_x + m_w  &&
               y > m_y        &&
               y < m_y + m_h
           );
}

bool Region::is_boundary( double x, double y ) const {
    return (
               x == m_x        ||
               x == m_x + m_w  ||
               y == m_y        ||
               y == m_y + m_h
           );
}

void Region::transform( const Cairo::Matrix& matrix ) {
    double x[4] = {m_x, m_x,     m_x+m_w, m_x+m_w};
    double y[4] = {m_y, m_y+m_h, m_y,     m_y+m_h};
    double xmin, xmax, ymin, ymax;

    for (int i=0; i < 4; i++)
        cairo_matrix_transform_point(&matrix, x+i, y+i);

    xmin = xmax = x[0];
    ymin = ymax = y[0];

    for (int i=1; i < 4; i++) {
        xmin = std::min(xmin, x[i]);
        xmax = std::max(xmax, x[i]);
        ymin = std::min(ymin, y[i]);
        ymax = std::max(ymax, y[i]);
    }

    m_x = xmin;
    m_y = ymin;
    m_w = xmax - xmin;
    m_h = ymax - ymin;

    m_signal_changed.emit();
}

double Region::left( )const {
    return m_x;
}

double Region::right( ) const {
    return m_x + m_w;
}

double Region::top( ) const {
    return m_y;
}

double Region::bottom( ) const {
    return m_y + m_h;
}

void Region::lrtb( double & l, double & r, double & t, double & b ) const {
    l = m_x;
    r = m_x + m_w;
    t = m_y;
    b = m_y + m_h;
}

void Region::center( double & x, double & y ) const {
    x = m_x + m_w / 2.0;
    y = m_y + m_h / 2.0;
}

}
