/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "selector.h"

#include <algorithm>

namespace Papyrus
{

  Selector::Selector( Group::pointer select_from_group, SelectionMode mode, int selection_depth ) :
      m_select_mode( mode ),
      m_select_from_group( select_from_group ),
      m_add_drawable_only( true ),
      m_selection_depth( selection_depth )
  {}

  Selector::Selector( Group::pointer select_from_group, Group::pointer selection_group, SelectionMode mode, int selection_depth, bool add_drawable_only ) :
      m_select_mode( mode ),
      m_select_from_group( select_from_group ),
      m_add_drawable_only( add_drawable_only ),
      m_selection_depth( selection_depth )
  {
    this->set_selection_group( selection_group );
  }

  Selector::pointer Selector::create( Group::pointer select_from_group, SelectionMode mode, int selection_depth )
  {
    return pointer( new Selector( select_from_group, mode, selection_depth ) );
  }

  Selector::pointer Selector::create( Group::pointer select_from_group, Group::pointer selection_group, SelectionMode mode, int selection_depth, bool add_drawable_only )
  {
    return pointer( new Selector( select_from_group, selection_group, mode, selection_depth, add_drawable_only ) );
  }

  Selector::~Selector()
  {}

  Group::pointer Selector::select_from_group()
  {
    return m_select_from_group;
  }

  void Selector::set_select_from_group( Group::pointer select_from_group )
  {
    this->clear();

    m_select_from_group = select_from_group;
  }

  Group::pointer Selector::selection_group()
  {
    return m_selection_group;
  }

  void Selector::set_selection_group( Group::pointer selection_group, bool add_drawable_only )
  {
    Selection::iterator i;
    for ( i = m_selection.begin(); i != m_selection.end(); i++ )
    {
      if ( m_selection_group )
        m_selection_group->remove( *i );
      if ( selection_group )
        selection_group->add( *i, 0, add_drawable_only );
    }
    m_selection_group = selection_group;
    m_add_drawable_only = add_drawable_only;
  }

  void Selector::set_select_mode( unsigned mode )
  {
    m_select_mode = mode;
  }

  void Selector::enable_select_mode( unsigned mode )
  {
    m_select_mode |= mode;
  }

  void Selector::disable_select_mode( unsigned mode )
  {
    m_select_mode &= ~mode;
  }

  unsigned Selector::select_mode( )
  {
    return m_select_mode;
  }

  int Selector::selection_depth()
  {
    return m_selection_depth;
  }

  void Selector::set_selection_depth( int selection_depth )
  {
    m_selection_depth = selection_depth;
  }

  void Selector::clear()
  {
    Drawable::pointer p;

    while ( m_selection.size() > 0 ) {
      p = m_selection.front();
      m_selection.pop_front();
      if ( m_selection_group )
        m_selection_group->remove( p );
      m_signal_deselected.emit( p );
    }

    m_signal_cleared.emit();
  }

  const Selector::Selection& Selector::selection()
  {
    return m_selection;
  }

  void Selector::add_to_selection( Drawable::pointer drawable )
  {
    if ( ! in_selection(drawable) ) {
      m_selection.push_back( drawable );
      if ( m_selection_group )
        m_selection_group->add( drawable, 0, m_add_drawable_only );
      m_signal_selected.emit( drawable );
    }
  }

  void Selector::remove_from_selection( Drawable::pointer drawable )
  {
    Selection::iterator found;
    found = std::find( m_selection.begin(), m_selection.end(), drawable );
    if ( found != m_selection.end() ) {
      m_selection.erase( found );
      if ( m_selection_group )
        m_selection_group->remove( drawable );
      m_signal_selected.emit( drawable );
    }
  }

  bool Selector::in_selection( Drawable::pointer drawable )
  {
    // TODO come back and add a hashed container to speed up lookup
    return ( std::find( m_selection.begin(), m_selection.end(), drawable ) != m_selection.end() );
  }

  const Region& Selector::extents( )
  {
    Selection::iterator i;

    if ( m_selection.size() == 0 )
      m_extents = Region();
    else
    {
      i = m_selection.begin();

      m_extents = (*i)->global_extents();
      while ( i != m_selection.end() )
      {
        m_extents.include( (*i)->global_extents() );
        i++;
      }
    }

    return m_extents;
  }

  sigc::signal<void, Drawable::pointer> Selector::signal_selected()
  {
    return m_signal_selected;
  }

  sigc::signal<void, Drawable::pointer> Selector::signal_deselected()
  {
    return m_signal_deselected;
  }

  sigc::signal<void> Selector::signal_cleared()
  {
    return m_signal_cleared;
  }

  bool Selector::on_button_press( const Event::ButtonPress& event )
  {
    std::vector<Drawable::pointer> results;
    std::vector<Drawable::pointer>::iterator i;

    if ( ! m_select_from_group )
      return false;

    if ( m_select_mode & SELECT_PRESS ) {

      results = m_select_from_group->select( event.x, event.y, m_selection_depth );

      if ( results.size() > 0 ) {

        // here, we have a selection made

        // if the CTRL key is pressed, or the selection size is 0, let's add everything to the selection
        if ( event.state & Event::CONTROL_MASK || m_selection.size() == 0 )
        {
          for ( i = results.begin(); i != results.end(); i++ )
            this->add_to_selection( *i );
        }
        else
        {
          // otherwise we have hit a situation where the control key is not pressed and we already
          // have some items selected
          // let's check and see if anything is already in the selection group
          bool existing = false;
          for ( i = results.begin(); i != results.end() && !existing; i++ )
            if ( this->in_selection(*i) )
              existing = true;

          // if we have a new selection, let's clear the old selection and start a new one,
          // otherwise we'll leave the current selection alone
          if ( !existing )
          {
            this->clear();
            for ( i = results.begin(); i != results.end(); i++ )
              this->add_to_selection( *i );
          }
        }

        m_last_position[ 0 ] = event.x;
        m_last_position[ 1 ] = event.y;

        return true;
      }
      else {
        // let's clear it if a blank area was selected and the CTRL key wasn't pressed
        if ( !( event.state & Event::CONTROL_MASK ) )
          this->clear();
      }
    }

    return false;
  }

  bool Selector::on_motion( const Event::Motion& event )
  {
    //       double dx, dy;
    //       dx = event->x;
    //         if (!m_line)
    //             return false;
    //
    //         if (sqrt(pow(m_last_position[0] - event->x, 2) + pow(m_last_position[1] - event->y, 2)) < m_movement_threshhold)
    //             return false;
    //
    //         m_line->add_vertex(event->x, event->y);
    //         m_last_position[0] = event->x;
    //         m_last_position[1] = event->y;

    return false;
  }


  bool Selector::on_button_release( const Event::ButtonRelease& event )
  {
    //         if (! (m_canvas && m_line) )
    //             return false;
    //
    //         m_canvas->remove
    //         (m_line);
    //         m_signal_complete(m_line);
    //         m_line.reset();

    return false;
  }

}
