/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#include "text.h"

namespace Papyrus {

    Text::Text( const std::string& text,
                double size,
                const std::string& family,
                Cairo::FontSlant slant,
                Cairo::FontWeight weight
              ):
      m_text(text),
      m_size(size),
      m_family(family),
      m_slant(slant),
      m_weight(weight) {}

    Text::pointer Text::create( const std::string& text,
                                double size,
                                const std::string& family,
                                Cairo::FontSlant slant,
                                Cairo::FontWeight weight
                              )
    {
        return Text::pointer( new Text(text, size, family, slant, weight) );
    }

    Text::~Text() {}

    const std::string& Text::font_family()
    {
      return m_family;
    }

    Cairo::FontSlant Text::font_slant()
    {
      return m_slant;
    }

    Cairo::FontWeight Text::font_weight()
    {
      return m_weight;
    }

    void Text::set_font_face( const std::string& family,
                              Cairo::FontSlant slant,
                              Cairo::FontWeight weight
                            )
    {
      m_family = family;
      m_slant = slant;
      m_weight = weight;
      this->shape_changed();
      m_signal_font.emit();
    }

    const Cairo::FontOptions& Text::font_options() {
        return m_font_options;
    }

    void Text::set_font_options( const Cairo::FontOptions& font_options) {
        m_font_options = font_options;
        this->shape_changed();
        m_signal_font.emit();
    }

    void Text::render( Cairo::RefPtr<Cairo::Context> cairo ) {
        Cairo::TextExtents extents;
        cairo->save();
        if ( m_family.size() > 0 )
          cairo->select_font_face( m_family, m_slant, m_weight );
        cairo->set_font_options(m_font_options);
        cairo->set_font_size(m_size);

        cairo->get_text_extents(m_text, extents);
        set_extents( Region(0.0, -extents.height, extents.width, extents.height) );
        cairo->restore();
        m_regenerate_matrix = true;
        Drawable::render(cairo);
    }

    double Text::size() {
        return m_size;
    }

    void Text::set_size(double size) {
        m_size = size;
        this->shape_changed();
        m_signal_font.emit();
    }

    std::string Text::text() const {
        return m_text;
    }

    void Text::set_text(const std::string text) {
        m_text = text;
        this->shape_changed();
        m_signal_text.emit();
    }

    void Text::draw_shape(Cairo::RefPtr<Cairo::Context> cairo) {
        Cairo::TextExtents extents;

        if ( m_family.size() > 0 )
          cairo->select_font_face( m_family, m_slant, m_weight );
        cairo->set_font_options(m_font_options);
        cairo->set_font_size(m_size);

        cairo->text_path(m_text);

        cairo->get_text_extents(m_text, extents);
        set_extents( Region(0.0, -extents.height, extents.width, extents.height) );
    }

    sigc::signal<void> Text::signal_text() {
        return m_signal_text;
    }

    sigc::signal<void> Text::signal_font() {
        return m_signal_font;
    }

}
